#include "pollutionError_konvex.h"
#include "mpi.h"
namespace AMDiS
{
	bool AMDiS::PollutionErrorKonvex::BoundaryTraverseStack::proofConditions(ElInfo *elInfo)
	{
	    if(elInfo==NULL) return false;
	    Element *curEl=elInfo->getElement();
	    PartitionElementData *ped=static_cast<PartitionElementData*>(curEl->getElementData(PARTITION_ED));
	    TEST_EXIT_DBG(ped!=NULL)("No PartitionElementData found");

	    if(ped!=NULL && ped->getPartitionStatus()==OVERLAP)
	    {
		//check neighbour
		int nrNeigh=curEl->getGeo(NEIGH);
		for(int i=0;i<nrNeigh;i++)
		{
		  Element* curNeigh=elInfo->getNeighbour(i);
		  if(curNeigh!=NULL) 
		  { 		    
		    ped=static_cast<PartitionElementData*>(curNeigh->getElementData(PARTITION_ED));
		    if(ped!=NULL && ped->getPartitionStatus()==ps) 
		    {
		      return true;
		    }
		  }
		}
	    }
	    return false;
	}

	ElInfo * AMDiS::PollutionErrorKonvex::BoundaryTraverseStack::traverseFirst(Mesh *mesh,PartitionStatus ps_)
	{
		FUNCNAME("PollutionErrorKonvex::BoundaryTraverseStack::traverseFirst");
		ElInfo *swap=TraverseStack::traverseFirst(mesh,-1, 
			      Mesh::CALL_LEAF_EL | Mesh::FILL_COORDS | Mesh::FILL_NEIGH);

		ps=ps_;
		if(swap!=NULL && !proofConditions(swap)) 
		  swap=traverseNext(swap);
	        return swap;
	}

	ElInfo * AMDiS::PollutionErrorKonvex::BoundaryTraverseStack::traverseNext(ElInfo *old)
	{
		FUNCNAME("PollutionErrorKonvex::BoundaryTraverseStack::traverseNext");
		ElInfo *swap=TraverseStack::traverseNext(old);
		while(swap!=NULL && !proofConditions(swap)) 
		{
		  swap=TraverseStack::traverseNext(swap);
		}
		return swap;
	}

  PollutionErrorKonvex::PollutionErrorKonvex(std::string name, int r):ResidualEstimator(name,r)
  {
      FUNCNAME("PollutionErrorKonvexEstimator::PollutionErrorKonvexEstimator()");
      GET_PARAMETER(0,name+"->C1p","%f",&C1p);
      GET_PARAMETER(0,name+"->C2p","%f",&C2p);

      /*C1=sqrt(C1);
      C2=sqrt(C2);*/
  }
  void PollutionErrorKonvex::init(double timestep)
  {
	FUNCNAME("PollutionErrorKonvex::init");
	ResidualEstimator::init(timestep);
	//set the distance (ie. width of the overlapped area)
	//approximate the overlapped area as rectangle. then set d=max(edge1, edge2)
	//retrieve all elements at the boundary overlapped -> out

	BoundaryTraverseStack* bts=new BoundaryTraverseStack();

	ElInfo* curEl=bts->traverseFirst(mesh,OUT);

	FixVec<WorldVector<double>, VERTEX> &fv=curEl->getCoords();
//std::cout<<"got fixvec"<<std::endl;
	WorldVector<double>* koordinaten=fv.begin();

	double xomin,xomax;
	xomin=xomax=(*koordinaten)[0];
	double yomin,yomax;
	yomin=yomax=(*koordinaten)[1];

	for(;koordinaten!=fv.end();koordinaten++)
	{
		xomin=min(xomin,(*koordinaten)[0]);
		xomax=max(xomax,(*koordinaten)[0]);
		yomin=min(yomin,(*koordinaten)[1]);
		yomax=max(yomax,(*koordinaten)[1]);
	}
	curEl=bts->traverseNext(curEl);
	//find xmax,xmin,ymax and ymin
	
	while(curEl!=NULL)
	{
		fv=curEl->getCoords();
		for(koordinaten=fv.begin();koordinaten!=fv.end();koordinaten++)
		{
			xomin=min(xomin,(*koordinaten)[0]);
			xomax=max(xomax,(*koordinaten)[0]);
			yomin=min(yomin,(*koordinaten)[1]);
			yomax=max(yomax,(*koordinaten)[1]);
		}
		curEl=bts->traverseNext(curEl);
	}
	//retrieve all elements at the boundary overlapped -> in
	curEl=bts->traverseFirst(mesh,OUT);

	fv=curEl->getCoords();
	koordinaten=fv.begin();
	double ximin,ximax;
	ximin=ximax=(*koordinaten)[0];
	double yimin,yimax;
	yimin=yimax=(*koordinaten)[1];
	for(;koordinaten!=fv.end();koordinaten++)
	{
		ximin=min(ximin,(*koordinaten)[0]);
		ximax=max(ximax,(*koordinaten)[0]);
		yimin=min(yimin,(*koordinaten)[1]);
		yimax=max(yimax,(*koordinaten)[1]);
	}
	curEl=bts->traverseNext(curEl);
	//find xmax,xmin,ymax and ymin
	while(curEl!=NULL)
	{
		fv=curEl->getCoords();
		for(koordinaten=fv.begin();koordinaten!=fv.end();koordinaten++)
		{
			ximin=min(ximin,(*koordinaten)[0]);
			ximax=max(ximax,(*koordinaten)[0]);
			yimin=min(yimin,(*koordinaten)[1]);
			yimax=max(yimax,(*koordinaten)[1]);
		}
		curEl=bts->traverseNext(curEl);
	}
	delete bts;
	double m1=max(abs(ximin-xomax),abs(xomin-ximax));
	double m2=max(abs(yimin-yomax),abs(yomin-yimax));
	d=min(m1,m2);
#if DEBUG
	std::cout<<"distance in polError:"<<d<<std::endl;
#endif
	C2p=C2p/d;
  }

  void PollutionErrorKonvex::estimateElement(ElInfo *elInfo)
  {
	FUNCNAME("PollutionErrorKonvex::estimateElement()");
    //switch between inner and outer
    //inner part --> use H_1Norm
    //outer part --> use L_2Norm
	PartitionElementData* ped=static_cast<PartitionElementData*>(elInfo->getElement()->getElementData(PARTITION_ED));
	TEST_EXIT_DBG(ped!=NULL)("No partition Elementdata found");
	PartitionStatus ps=ped->getPartitionStatus();
	TEST_EXIT_DBG(ps!=UNDEFINED)("Elementstatus undefined");
	//save old values
	double C0,C1;
	C0=ResidualEstimator::C0;
	C1=ResidualEstimator::C1;
	if(ps==OUT)
	{
		//outer part
		ResidualEstimator::norm=L2_NORM;
		
		ResidualEstimator::C0 *=C2p;
		ResidualEstimator::C1 *=C2p;		
		
	}else
	{
		//inner part
		ResidualEstimator::norm=H1_NORM;
		ResidualEstimator::C0 *=C1p;
		ResidualEstimator::C1 *=C1p;
	}
	ResidualEstimator::estimateElement(elInfo);
	//set old values
	ResidualEstimator::C0=C0;
	ResidualEstimator::C1=C1;
  }

//   void exit(bool output=true)
//   {
//     
//   }
}

